@@ -30,6 +30,8 @@ require 'rbot/channel'
 require 'rbot/utils'
 require 'rbot/message'
 require 'rbot/language'
+require 'rbot/dbhash'
+require 'rbot/registry'
 
 module Irc
 
@@ -54,25 +56,41 @@ class IrcBot
   
   # bot's Language data
   attr_reader :lang
+
+  # bot's configured addressing prefixes
+  attr_reader :addressing_prefixes
+
+  # bot's object registry, plugins get an interface to this for persistant
+  # storage (hash interface tied to a bdb file, plugins use Accessors to store
+  # and restore objects in their own namespaces.)
+  attr_reader :registry
   
   # create a new IrcBot with botclass +botclass+
   def initialize(botclass)
-    @botclass = botclass
-    @botclass.gsub!(/\/$/, "")
+    @botclass = botclass.gsub(/\/$/, "")
+    
     Dir.mkdir("#{botclass}") if(!File.exist?("#{botclass}"))
     Dir.mkdir("#{botclass}/logs") if(!File.exist?("#{botclass}/logs"))
+    
     @config = Irc::BotConfig.new(self)
     @timer = Timer::Timer.new
+    @registry = BotRegistry.new self
     @timer.add(@config["SAVE_EVERY"].to_i) { save }
     @channels = Hash.new
     @logs = Hash.new
+    
+    @lang = Irc::Language.new(@config["LANGUAGE"])
     @keywords = Irc::Keywords.new(self)
     @auth = Irc::IrcAuth.new(self)
-    @lang = Irc::Language.new(@config["LANGUAGE"])
     @plugins = Irc::Plugins.new(self, ["#{botclass}/plugins"])
-    @socket = Irc::IrcSocket.new(@config["SERVER"], @config["PORT"], @config["HOST"])
+    @socket = Irc::IrcSocket.new(@config["SERVER"], @config["PORT"], @config["HOST"], @config["SENDQ_DELAY"], @config["SENDQ_BURST"])
     @nick = @config["NICK"]
     @server_password = @config["SERVER_PASSWORD"]
+    if @config["ADDRESS_PREFIX"]
+      @addressing_prefixes = @config["ADDRESS_PREFIX"].split(" ")
+    else
+      @addressing_prefixes = Array.new
+    end
     
     @client = Irc::IrcClient.new
     @client["PRIVMSG"] = proc { |data|
@@ -96,11 +114,13 @@ class IrcBot
       puts "WARNING, bad nick (#{data['NICK']})"
     }
     @client["PING"] = proc {|data|
+      # (jump the queue for pongs)
       @socket.puts "PONG #{data['PINGID']}"
     }
     @client["NICK"] = proc {|data|
       sourcenick = data["SOURCENICK"]
       nick = data["NICK"]
+      m = NickMessage.new(self, data["SOURCE"], data["SOURCENICK"], data["NICK"])
       if(sourcenick == @nick)
         @nick = nick
       end
@@ -111,12 +131,15 @@ class IrcBot
           v.users.delete(sourcenick)
         end
       }
+      @plugins.delegate("listen", m)
+      @plugins.delegate("nick", m)
     }
     @client["QUIT"] = proc {|data|
       source = data["SOURCE"]
       sourcenick = data["SOURCENICK"]
       sourceurl = data["SOURCEADDRESS"]
       message = data["MESSAGE"]
+      m = QuitMessage.new(self, data["SOURCE"], data["SOURCENICK"], data["MESSAGE"])
       if(data["SOURCENICK"] =~ /#{@nick}/)
       else
         @channels.each {|k,v|
@@ -126,6 +149,8 @@ class IrcBot
           end
         }
       end
+      @plugins.delegate("listen", m)
+      @plugins.delegate("quit", m)
     }
     @client["MODE"] = proc {|data|
       source = data["SOURCE"]
@@ -177,6 +202,9 @@ class IrcBot
       else
         log "@ #{sourcenick} set topic \"#{topic}\"", channel
       end
+      m = TopicMessage.new(self, data["SOURCE"], data["CHANNEL"], data["TOPIC"])
+      @plugins.delegate("topic", m)
+      @plugins.delegate("listen", m)
     }
     @client["NAMES"] = proc {|data|
       channel = data["CHANNEL"]
@@ -202,7 +230,7 @@ class IrcBot
     trap("SIGINT") { quit }
     begin
       @socket.connect
-    rescue => e
+      rescue => e
       raise "failed to connect to IRC server at #{@config['SERVER']} #{@config['PORT']}: " + e
     end
     @socket.puts "PASS " + @server_password if @server_password
@@ -217,16 +245,21 @@ class IrcBot
     while true
       connect
       
-      while true
-        if @socket.select socket_timeout
-          break unless reply = @socket.gets
-          @client.process reply
+      begin
+        while true
+          if @socket.select socket_timeout
+            break unless reply = @socket.gets
+            @client.process reply
+          end
+          @timer.tick
         end
-        @timer.tick
+      rescue => e
+        puts "connection closed: #{e}"
       end
       
       puts "disconnected"
       @channels.clear
+      @socket.clearq
       
       puts "waiting to reconnect"
       sleep reconnect_wait
@@ -243,24 +276,28 @@ class IrcBot
   def sendmsg(type, where, message)
     # limit it 440 chars + CRLF.. so we have to split long lines
     left = 440 - type.length - where.length - 3
-    if(left >= message.length)
-      log_sent(type, where, message)
-      @socket.puts("#{type} #{where} :#{message}")
-    else
-      debug "Line too long, splitting"
-      begin
-        line = message.slice!(0, left)
-        lastspace = line.rindex(/\s+/)
-        if(lastspace)
-          message = line.slice!(lastspace, line.length) + message
-          message.gsub!(/^\s+/, "")
-        end
-        @socket.puts("#{type} #{where} :#{line}")
-        log_sent(type, where, line)
-      end while(message.length > 0)
-    end
+    begin
+      if(left >= message.length)
+        sendq("#{type} #{where} :#{message}")
+        log_sent(type, where, message)
+        return
+      end
+      line = message.slice!(0, left)
+      lastspace = line.rindex(/\s+/)
+      if(lastspace)
+        message = line.slice!(lastspace, line.length) + message
+        message.gsub!(/^\s+/, "")
+      end
+      sendq("#{type} #{where} :#{line}")
+      log_sent(type, where, line)
+    end while(message.length > 0)
   end
-  
+
+  def sendq(message="")
+    # temporary
+    @socket.queue(message)
+  end
+
   # send a notice message to channel/nick +where+
   def notice(where, message)
     message.each_line { |line|
@@ -272,7 +309,7 @@ class IrcBot
 
   # say something (PRIVMSG) to channel/nick +where+
   def say(where, message)
-    message.each_line { |line|
+    message.to_s.each_line { |line|
       line.chomp!
       next unless(line.length > 0)
       unless((where =~ /^#/) && (@channels.has_key?(where) && @channels[where].quiet))
@@ -283,7 +320,7 @@ class IrcBot
 
   # perform a CTCP action with message +message+ to channel/nick +where+
   def action(where, message)
-    @socket.puts("PRIVMSG #{where} :\001ACTION #{message}\001")
+    sendq("PRIVMSG #{where} :\001ACTION #{message}\001")
     if(where =~ /^#/)
       log "* #{@nick} #{message}", where
     elsif (where =~ /^(\S*)!.*$/)
@@ -308,32 +345,38 @@ class IrcBot
       @logs[where].sync = true
     end
     @logs[where].puts "[#{stamp}] #{message}"
-    debug "[#{stamp}] <#{where}> #{message}"
+    #debug "[#{stamp}] <#{where}> #{message}"
   end
   
   # set topic of channel +where+ to +topic+
   def topic(where, topic)
-    @socket.puts "TOPIC #{where} :#{topic}"
+    sendq "TOPIC #{where} :#{topic}"
   end
   
   # message:: optional IRC quit message
   # quit IRC, shutdown the bot
-  def quit(message="bye")
-    message="bye" if (!message || message.length < 1)
-    @socket.puts "QUIT :#{message}"
-    @socket.flush
-    @socket.shutdown
+  def quit(message=nil)
+    trap("SIGTERM", "DEFAULT")
+    trap("SIGHUP", "DEFAULT")
+    trap("SIGINT", "DEFAULT")
+    message = @lang.get("quit") if (!message || message.length < 1)
+    @socket.clearq
+    save
+    @plugins.cleanup
     @channels.each_value {|v|
       log "@ quit (#{message})", v.name
     }
-    save
-    @plugins.cleanup
+    @socket.puts "QUIT :#{message}"
+    @socket.flush
+    @socket.shutdown
+    @registry.close
     puts "rbot quit (#{message})"
     exit 0
   end
 
   # call the save method for bot's config, keywords, auth and all plugins
   def save
+    @registry.flush
     @config.save
     @keywords.save
     @auth.save
@@ -352,20 +395,20 @@ class IrcBot
   # join a channel
   def join(channel, key=nil)
     if(key)
-      @socket.puts "JOIN :#{channel} #{key}"
+      sendq "JOIN #{channel} :#{key}"
     else
-      @socket.puts "JOIN :#{channel}"
+      sendq "JOIN #{channel}"
     end
   end
 
   # part a channel
-  def part(channel)
-    @socket.puts "PART :#{channel}"
+  def part(channel, message="")
+    sendq "PART #{channel} :#{message}"
   end
 
   # attempt to change bot's nick to +name+
   def nickchg(name)
-      @socket.puts "NICK #{name}"
+      sendq "NICK #{name}"
   end
   
   # m::     message asking for help
@@ -374,26 +417,26 @@ class IrcBot
   def help(topic=nil)
     topic = nil if topic == ""
     case topic
-      when nil
-        helpstr = "rbot help topics: core, auth, keywords"
-        helpstr += @plugins.helptopics
-        helpstr += " (help <topic> for more info)"
-      when /^core$/
-        helpstr = corehelp
-      when /^core\s+(.+)$/
-        helpstr = corehelp $1
-      when /^auth$/
-        helpstr = @auth.help
-      when /^auth\s+(.+)$/
-        helpstr = @auth.help $1
-      when /^keywords$/
-        helpstr = @keywords.help
-      when /^keywords\s+(.+)$/
-        helpstr = @keywords.help $1
-      else
-        unless(helpstr = @plugins.help(topic))
-          helpstr = "no help for topic #{topic}"
-        end
+    when nil
+      helpstr = "help topics: core, auth, keywords"
+      helpstr += @plugins.helptopics
+      helpstr += " (help <topic> for more info)"
+    when /^core$/
+      helpstr = corehelp
+    when /^core\s+(.+)$/
+      helpstr = corehelp $1
+    when /^auth$/
+      helpstr = @auth.help
+    when /^auth\s+(.+)$/
+      helpstr = @auth.help $1
+    when /^keywords$/
+      helpstr = @keywords.help
+    when /^keywords\s+(.+)$/
+      helpstr = @keywords.help $1
+    else
+      unless(helpstr = @plugins.help(topic))
+        helpstr = "no help for topic #{topic}"
+      end
     end
     return helpstr
   end
@@ -441,7 +484,7 @@ class IrcBot
   # handle incoming IRC PRIVMSG +m+
   def onprivmsg(m)
     # log it first
-    if(m.message =~ /^\001ACTION\s+(.+)\001/)
+    if(m.action?)
       if(m.private?)
         log "* [#{m.sourcenick}(#{m.sourceaddress})] #$1", m.sourcenick
       else
@@ -467,7 +510,7 @@ class IrcBot
     if(m.address?)
       case m.message
         when (/^join\s+(\S+)\s+(\S+)$/)
-          join $1, $2 if(@auth.allow?("join", m.source, m.replytok))
+          join $1, $2 if(@auth.allow?("join", m.source, m.replyto))
         when (/^join\s+(\S+)$/)
           join $1 if(@auth.allow?("join", m.source, m.replyto))
         when (/^part$/)
@@ -522,6 +565,33 @@ class IrcBot
             @channels[where].quiet = false if(@channels.has_key?(where))
             okay m.replyto
           end
+        # TODO break this out into an options module
+        when (/^options get sendq_delay$/)
+          if auth.allow?("config", m.source, m.replyto)
+            m.reply "options->sendq_delay = #{@socket.get_sendq}"
+          end
+        when (/^options get sendq_burst$/)
+          if auth.allow?("config", m.source, m.replyto)
+            m.reply "options->sendq_burst = #{@socket.get_maxburst}"
+          end
+        when (/^options set sendq_burst (.*)$/)
+          num = $1.to_i
+          if auth.allow?("config", m.source, m.replyto)
+            @socket.set_maxburst(num)
+            @config["SENDQ_BURST"] = num
+            okay m.replyto
+          end
+        when (/^options set sendq_delay (.*)$/)
+          freq = $1.to_f
+          if auth.allow?("config", m.source, m.replyto)
+            @socket.set_sendq(freq)
+            @config["SENDQ_DELAY"] = freq
+            okay m.replyto
+          end
+        when (/^registry stats$/)
+          if auth.allow?("config", m.source, m.replyto)
+            m.reply @registry.stat.inspect
+          end
         when (/^(version)|(introduce yourself)$/)
           say m.replyto, "I'm a v. #{$version} rubybot, (c) Tom Gilbert - http://linuxbrit.co.uk/rbot/"
         when (/^help(?:\s+(.*))?$/)
@@ -538,7 +608,7 @@ class IrcBot
     else
       # stuff to handle when not addressed
       case m.message
-        when (/^\s*(hello|howdy|hola|salut|bonjour|sup|niihau|hey|hi(\W|$)|yo(\W|$)).*#{@nick}/)
+        when (/^\s*(hello|howdy|hola|salut|bonjour|sup|niihau|hey|hi(\W|$)|yo(\W|$))\s+#{@nick}$/)
           say m.replyto, @lang.get("hello_X") % m.sourcenick
         when (/^#{@nick}!*$/)
           say m.replyto, @lang.get("hello_X") % m.sourcenick
@@ -581,7 +651,7 @@ class IrcBot
       @channels[m.channel].users[m.sourcenick]["mode"] = ""
     end
 
-    # delegate to plugins
+    @plugins.delegate("listen", m)
     @plugins.delegate("join", m)
   end
 
@@ -596,6 +666,7 @@ class IrcBot
     end
     
     # delegate to plugins
+    @plugins.delegate("listen", m)
     @plugins.delegate("part", m)
   end
 
@@ -606,11 +677,11 @@ class IrcBot
       log "@ You have been kicked from #{m.channel} by #{m.sourcenick} (#{m.message})", m.channel
       puts "kicked from channel #{m.channel}"
     else
-      @channels[channel].users.delete(m.sourcenick)
+      @channels[m.channel].users.delete(m.sourcenick)
       log "@ #{m.target} has been kicked from #{m.channel} by #{m.sourcenick} (#{m.message})", m.channel
     end
 
-    # delegate to plugins
+    @plugins.delegate("listen", m)
     @plugins.delegate("kick", m)
   end
 
