@@ -11,8 +11,16 @@ class HostPlugin < Plugin
       if(p)
         m.reply p.readlines.join("\n")
       else
-        $stderr = $stdout
-        exec("host", m.params)
+        begin
+          $stderr = $stdout
+          exec("host", m.params)
+          Kernel::exit! 0
+        rescue Exception => e
+          puts "host plugin exception: #{e}"
+          Kernel::exit! 0
+        end
+        puts "exec of host failed"
+        Kernel::exit! 0
       end
     }
     return
