@@ -857,178 +857,156 @@ module Irc
     def process(serverstring)
       data = Hash.new
       data["SERVERSTRING"] = serverstring
-      case serverstring
-         when (/^PING\s+(\S+)/)
-           data["PINGID"] = $1
-           handle("PING", data)
-         when (/^:(\S+)\s+(\d+)\s+(.*)/)
-           # numeric server message
-           data["SOURCE"] = $1
-           num=$2.to_i
-           params=$3
-           case num
-             when ERR_NICKNAMEINUSE
-               # "* <nick> :Nickname is already in use"
-               if(params =~ /^\s*\*\s+(\S+)\s+:?(.*)/)
-                 data["NICK"] = $1
-                 data["MESSAGE"] = $2
-               end
-               handle("NICKTAKEN", data)
-             when ERR_ERRONEUSNICKNAME
-               # "* <nick> :Erroneous nickname"
-               if(params =~ /^\s*\*\s+(\S+)\s+:?(.*)/)
-                 data["NICK"] = $1
-                 data["MESSAGE"] = $2
-               end
-               handle("BADNICK", data)
-             when RPL_TOPIC
-               if(params =~ /^(\S+)\s+:(.*)$/)
-                 data["CHANNEL"] = $1
-                 data["TOPIC"] = $2
-               end
-               handle("TOPIC", data)
-             when RPL_TOPIC_INFO
-               if(params =~ /^(\S+)\s+:(.*)$/)
-                 data["CHANNEL"] = $1
-                 data["NICK"] = $2
-                 data["UNIXTIME"] = $3
-               end
-               handle("TOPICINFO", data)
-             when RPL_NAMREPLY
-               # "( "=" / "*" / "@" ) <channel>
-               # :[ "@" / "+" ] <nick> *( " " [ "@" / "+" ] <nick> )
-               # - "@" is used for secret channels, "*" for private
-               # channels, and "=" for others (public channels).
-               if(params =~ /^(\S+)\s+([=*@])\s+(\S+)\s+:(.*)$/)
-                 $4.split(" ").each {|u|
-                   if(u =~ /^([@+])(.*)$/)
-                     umode = $1
-                     user = $2
-                     @users << [user, umode]
-                   else
-                     @users << [u, ""]
-                   end
-                 }
-               end
-             when RPL_ENDOFNAMES
-               if(params =~ /^(\S+)\s+(\S+)\s+.*$/)
-                 data["CHANNEL"] = $2
-                 data["USERS"] = @users
-                 handle("NAMES", data)
-               end
-               @users = Array.new
-             when RPL_WELCOME
-               # "Welcome to the Internet Relay Network
-               # <nick>!<user>@<host>"
-               if(params =~ /((\S+)!(\S+))/)
-                 data["NETMASK"] = $1
-                 data["NICK"] = $2
-                 data["ADDRESS"] = $3
-               elsif(params =~ /Welcome to the Internet Relay Network\s(\S+)/)
-                 data["NICK"] = $1
-               elsif(params =~ /Welcome.*Network\s+(\S+)/)
-                 data["NICK"] = $1
-               end
-               handle("WELCOME", data)
-             when RPL_MOTDSTART
-               # "<nick> :- <server> Message of the Day -"
-               if(params =~ /^(\S+)\s+:-\s+(\S+)\s+(.*)$/)
-                 server = $2
-                 @motd = ""
-               end
-             when RPL_MOTD
-               if(params =~ /^(\S+)\s+:-\s+(.*)$/)
-                 @motd += $2 + "\n"
-               end
-             when RPL_ENDOFMOTD
-               data["MOTD"] = @motd
-               handle("MOTD", data)
-             else
-               handle("UNKNOWN", data)
-           end
-         when (/^:((\S+)!(\S+))\s+PRIVMSG\s+(\S+)\s+:?(.*)/)
-           # you can either bind to "PRIVMSG", to get every one and
-           # parse it yourself, or you can bind to "MSG", "PUBLIC",
-           # etc and get it all nicely split up for you.
-           data["SOURCE"] = $1
-           data["SOURCENICK"] = $2
-           data["SOURCEADDRESS"] = $3
-           data["TARGET"] = $4
-           data["MESSAGE"] = $5
-           handle("PRIVMSG", data)
+      
+      unless serverstring =~ /^(:(\S+)\s)?(\S+)(\s(.*))?/
+	      raise "Unparseable Server Message!!!: #{serverstring}"
+      end
+ 
+      prefix, command, params = $2, $3, $5
+      
+      if prefix != nil
+        data['SOURCE'] = prefix
+        if prefix =~ /^(\S+)!(\S+)$/
+          data['SOURCENICK'] = $1
+          data['SOURCEADDRESS'] = $2
+        end
+      end
 
-           # Now we split it
-           if(data["TARGET"] =~ /^(#|&).*/)
-             handle("PUBLIC", data)
-           else
-             handle("MSG", data)
-           end
-         when (/^:((\S+)!(\S+))\s+KICK\s+(\S+)\s+(\S+)\s+:?(.*)/)
-           data["SOURCE"] = $1
-           data["SOURCENICK"] = $2
-           data["SOURCEADDRESS"] = $3
-           data["CHANNEL"] = $4
-           data["TARGET"] = $5
-           data["MESSAGE"] = $6
-           handle("KICK", data)
-         when (/^:((\S+)!(\S+))\s+PART\s+(\S+)\s+:?(.*)/)
-           data["SOURCE"] = $1
-           data["SOURCENICK"] = $2
-           data["SOURCEADDRESS"] = $3
-           data["CHANNEL"] = $4
-           data["MESSAGE"] = $5
-           handle("PART", data)
-         when (/^:((\S+)!(\S+))\s+QUIT\s+:?(\S+)/)
-           data["SOURCE"] = $1
-           data["SOURCENICK"] = $2
-           data["SOURCEADDRESS"] = $3
-           data["MESSAGE"] = $4
-           handle("QUIT", data)
-         when (/^:((\S+)!(\S+))\s+JOIN\s+:?(\S+)/)
-           data["SOURCE"] = $1
-           data["SOURCENICK"] = $2
-           data["SOURCEADDRESS"] = $3
-           data["CHANNEL"] = $4
-           handle("JOIN", data)
-         when (/^:((\S+)!(\S+))\s+TOPIC\s+(\S+)\s+:(.*)$/)
-           data["SOURCE"] = $1
-           data["SOURCENICK"] = $2
-           data["SOURCEADDRESS"] = $3
-           data["CHANNEL"] = $4
-           data["TOPIC"] = $5
-           handle("CHANGETOPIC", data)
-         when (/^:((\S+)!(\S+))\s+INVITE\s+(\S+)\s+:(\S+)$/)
-           data["SOURCE"] = $1
-           data["SOURCENICK"] = $2
-           data["SOURCEADDRESS"] = $3
-           data["TARGET"] = $4
-           data["CHANNEL"] = $5
-           handle("INVITE", data)
-         when (/^:((\S+)!(\S+))\s+NICK\s+:(\S+)$/)
-           data["SOURCE"] = $1
-           data["SOURCENICK"] = $2
-           data["SOURCEADDRESS"] = $3
-           data["NICK"] = $4
-           handle("NICK", data)
-         when (/^:((\S+)!(\S+))\s+MODE\s+(\S+)\s+(\S+)\s+(.*)$/)
-           data["SOURCE"] = $1
-           data["SOURCENICK"] = $2
-           data["SOURCEADDRESS"] = $3
-           data["CHANNEL"] = $4
-           data["MODESTRING"] = $5
-           data["TARGETS"] = $6
-           handle("MODE", data)
-         when (/^:(\S+)\s+NOTICE\s+(\S+)\s+:?(.*)/)
-           data["SOURCE"] = $1
-           data["TARGET"] = $2
-           data["MESSAGE"] = $3
-           if($1 =~ /(\S+)!(\S+)/)
-             data["SOURCENICK"] = $1
-             data["SOURCEADDRESS"] = $2
-           end
-           handle("NOTICE", data)
-         else
-           handle("UNKNOWN", data)
+      # split parameters in an array
+      argv = []
+      params.scan(/(?!:)(\S+)|:(.*)/) { argv << ($1 || $2) } if params
+      
+      case command
+      when 'PING'
+        data['PINGID'] = argv[0]
+        handle('PING', data)
+      when /^(\d+)$/		# numeric server message
+        num=command.to_i
+        case num
+        when ERR_NICKNAMEINUSE
+          # "* <nick> :Nickname is already in use"
+          data['NICK'] = argv[1]
+          data['MESSAGE'] = argv[2]
+          handle('NICKTAKEN', data)
+        when ERR_ERRONEUSNICKNAME
+          # "* <nick> :Erroneous nickname"
+          data['NICK'] = argv[1]
+          data['MESSAGE'] = argv[2]
+          handle('BADNICK', data)
+        when RPL_TOPIC
+          data['CHANNEL'] = argv[0]
+          data['TOPIC'] = argv[1]
+          handle('TOPIC', data)
+        when RPL_TOPIC_INFO
+          data['CHANNEL'] = argv[0]
+          data['NICK'] = argv[1]
+          data['UNIXTIME'] = argv[2]
+          handle('TOPICINFO', data)
+        when RPL_NAMREPLY
+          # "( "=" / "*" / "@" ) <channel>
+          # :[ "@" / "+" ] <nick> *( " " [ "@" / "+" ] <nick> )
+          # - "@" is used for secret channels, "*" for private
+          # channels, and "=" for others (public channels).
+          argv[3..-1].each { |u|
+            if(u =~ /^([@+])?(.*)$/)
+              umode = $1 || ""
+              user = $2
+              @users << [user, umode]
+            end
+          }
+        when RPL_ENDOFNAMES
+          data['CHANNEL'] = argv[1]
+          data['USERS'] = @users
+          handle('NAMES', data)
+          @users = Array.new
+        when RPL_WELCOME
+          # "Welcome to the Internet Relay Network
+          # <nick>!<user>@<host>"
+          case argv[1]
+          when /((\S+)!(\S+))/
+            data['NETMASK'] = $1
+            data['NICK'] = $2
+            data['ADDRESS'] = $3
+          when /Welcome to the Internet Relay Network\s(\S+)/
+            data['NICK'] = $1
+          when /Welcome.*Network\s+(\S+)/
+            data['NICK'] = $1
+          end
+          handle('WELCOME', data)
+        when RPL_MOTDSTART
+          # "<nick> :- <server> Message of the Day -"
+          if argv[1] =~ /^-\s+(\S+)\s/
+            server = $1
+            @motd = ""
+          end
+        when RPL_MOTD
+          if(argv[1] =~ /^-\s+(.*)$/)
+            @motd << $1
+            @motd << "\n"
+          end
+        when RPL_ENDOFMOTD
+          data['MOTD'] = @motd
+          handle('MOTD', data)
+        else
+          handle('UNKNOWN', data)
+        end
+      # end of numeric replies
+      when 'PRIVMSG'
+        # you can either bind to 'PRIVMSG', to get every one and
+        # parse it yourself, or you can bind to 'MSG', 'PUBLIC',
+        # etc and get it all nicely split up for you.
+        data['TARGET'] = argv[0]
+        data['MESSAGE'] = argv[1]
+        handle('PRIVMSG', data)
+        
+        # Now we split it
+        if(data['TARGET'] =~ /^(#|&).*/)
+          handle('PUBLIC', data)
+        else
+          handle('MSG', data)
+        end
+      when 'KICK'
+        data['CHANNEL'] = argv[0]
+        data['TARGET'] = argv[1]
+        data['MESSAGE'] = argv[2]
+        handle('KICK', data)
+      when 'PART'
+        data['CHANNEL'] = argv[0]
+        data['MESSAGE'] = argv[1]
+        handle('PART', data)
+      when 'QUIT'
+        data['MESSAGE'] = argv[0]
+        handle('QUIT', data)
+      when 'JOIN'
+        data['CHANNEL'] = argv[0]
+        handle('JOIN', data)
+      when 'TOPIC'
+        data['CHANNEL'] = argv[0]
+        data['TOPIC'] = argv[1]
+        handle('CHANGETOPIC', data)
+      when 'INVITE'
+        data['TARGET'] = argv[0]
+        data['CHANNEL'] = argv[1]
+        handle('INVITE', data)
+      when 'NICK'
+        data['NICK'] = argv[0]
+        handle('NICK', data)
+      when 'MODE'
+        data['CHANNEL'] = argv[0]
+        data['MODESTRING'] = argv[1]
+        data['TARGETS'] = argv[2]
+        handle('MODE', data)
+      when 'NOTICE'
+        data['TARGET'] = argv[0]
+        data['MESSAGE'] = argv[1]
+        if data['SOURCENICK']
+          handle('NOTICE', data)
+        else
+          # "server notice" (not from user, noone to reply to
+          handle('SNOTICE', data)
+        end
+      else
+        handle('UNKNOWN', data)
       end
     end
 
